package edu.uci.ics.genomix.pregelix.operator.pathmerge;

import java.io.IOException;
import java.util.Iterator;

import org.apache.hadoop.io.NullWritable;


import edu.uci.ics.pregelix.api.graph.Vertex;
import edu.uci.ics.genomix.pregelix.io.MessageWritable;
import edu.uci.ics.genomix.pregelix.io.VertexValueWritable;
import edu.uci.ics.genomix.pregelix.io.VertexValueWritable.State;
import edu.uci.ics.genomix.pregelix.type.AdjMessage;
import edu.uci.ics.genomix.pregelix.type.MessageFlag;
import edu.uci.ics.genomix.pregelix.util.VertexUtil;
import edu.uci.ics.genomix.type.GeneCode;
import edu.uci.ics.genomix.type.KmerBytesWritable;

/**
 * Naive Algorithm for path merge graph
 */
public class BasicGraphCleanVertex extends
        Vertex<KmerBytesWritable, VertexValueWritable, NullWritable, MessageWritable> {
    public static final String KMER_SIZE = "BasicGraphCleanVertex.kmerSize";
    public static final String ITERATIONS = "BasicGraphCleanVertex.iteration";
    public static int kmerSize = -1;
    protected int maxIteration = -1;
    
    protected MessageWritable incomingMsg = null; 
    protected MessageWritable outgoingMsg = null; 
    protected KmerBytesWritable destVertexId = new KmerBytesWritable();
    protected Iterator<KmerBytesWritable> posIterator;
    protected KmerBytesWritable tmpKmer = new KmerBytesWritable(kmerSize);
    byte headFlag;
    protected byte outFlag;
    protected byte inFlag;
    protected byte selfFlag;
    
    /**
     * initiate kmerSize, maxIteration
     */
    public void initVertex() {
    }
    
    /**
     * reset headFlag
     */
    public void resetHeadFlag(){
        headFlag = (byte)(getVertexValue().getState() & State.IS_HEAD);
    }
    
    public byte getHeadFlag(){
        return (byte)(getVertexValue().getState() & State.IS_HEAD);
    }
    
    /**
     * check if prev/next destination exists
     */
    public boolean hasNextDest(VertexValueWritable value){
        return value.getFFList().getCountOfPosition() > 0 || value.getFRList().getCountOfPosition() > 0;
    }
    
    public boolean hasPrevDest(VertexValueWritable value){
        return value.getRFList().getCountOfPosition() > 0 || value.getRRList().getCountOfPosition() > 0;
    }
    
    /**
     * get destination vertex
     */
    public KmerBytesWritable getNextDestVertexId(VertexValueWritable value) {
        if (value.getFFList().getCountOfPosition() > 0){ // #FFList() > 0
            posIterator = value.getFFList().iterator();
            return posIterator.next();
        } else if (value.getFRList().getCountOfPosition() > 0){ // #FRList() > 0
            posIterator = value.getFRList().iterator();
            return posIterator.next();
        } else {
            return null;  
        }
    }

    public KmerBytesWritable getPreDestVertexId(VertexValueWritable value) {
        if (value.getRFList().getCountOfPosition() > 0){ // #RFList() > 0
            posIterator = value.getRFList().iterator();
            return posIterator.next();
        } else if (value.getRRList().getCountOfPosition() > 0){ // #RRList() > 0
            posIterator = value.getRRList().iterator();
            return posIterator.next();
        } else {
            return null;
        }
    }
    
    /**
     * get destination vertex
     */
    public KmerBytesWritable getNextDestVertexIdAndSetFlag(VertexValueWritable value) {
        if (value.getFFList().getCountOfPosition() > 0){ // #FFList() > 0
            posIterator = value.getFFList().iterator();
            outFlag &= MessageFlag.DIR_CLEAR;
            outFlag |= MessageFlag.DIR_FF;
            return posIterator.next();
        } else if (value.getFRList().getCountOfPosition() > 0){ // #FRList() > 0
            posIterator = value.getFRList().iterator();
            outFlag &= MessageFlag.DIR_CLEAR;
            outFlag |= MessageFlag.DIR_FR;
            return posIterator.next();
        } else {
          return null;  
        }
        
    }

    public KmerBytesWritable getPreDestVertexIdAndSetFlag(VertexValueWritable value) {
        if (value.getRFList().getCountOfPosition() > 0){ // #RFList() > 0
            posIterator = value.getRFList().iterator();
            outFlag &= MessageFlag.DIR_CLEAR;
            outFlag |= MessageFlag.DIR_RF;
            return posIterator.next();
        } else if (value.getRRList().getCountOfPosition() > 0){ // #RRList() > 0
            posIterator = value.getRRList().iterator();
            outFlag &= MessageFlag.DIR_CLEAR;
            outFlag |= MessageFlag.DIR_RR;
            return posIterator.next();
        } else {
            return null;
        }
    }

    /**
     * head send message to all next nodes
     */
    public void sendMsgToAllNextNodes(VertexValueWritable value) {
        posIterator = value.getFFList().iterator(); // FFList
        while(posIterator.hasNext()){
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
        posIterator = value.getFRList().iterator(); // FRList
        while(posIterator.hasNext()){
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
    }

    /**
     * head send message to all previous nodes
     */
    public void sendMsgToAllPreviousNodes(VertexValueWritable value) {
        posIterator = value.getRFList().iterator(); // RFList
        while(posIterator.hasNext()){
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
        posIterator = value.getRRList().iterator(); // RRList
        while(posIterator.hasNext()){
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
    }

    /**
     * one vertex send message to previous and next vertices (neighbor)
     */
    public void sendMsgToAllNeighborNodes(VertexValueWritable value){
        sendMsgToAllNextNodes(value);
        sendMsgToAllPreviousNodes(value);
    }
    
    /**
     * head send message to all previous nodes
     */
    public void sendSettledMsgToAllPreviousNodes(VertexValueWritable value) {
        posIterator = value.getRFList().iterator(); // RFList
        while(posIterator.hasNext()){
            outgoingMsg.setFlag(AdjMessage.FROMRF);
            outgoingMsg.setSourceVertexId(getVertexId());
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
        posIterator = value.getRRList().iterator(); // RRList
        while(posIterator.hasNext()){
            outgoingMsg.setFlag(AdjMessage.FROMRR);
            outgoingMsg.setSourceVertexId(getVertexId());
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
    }
    
    /**
     * head send message to all next nodes
     */
    public void sendSettledMsgToAllNextNodes(VertexValueWritable value) {
        posIterator = value.getFFList().iterator(); // FFList
        while(posIterator.hasNext()){
            outgoingMsg.setFlag(AdjMessage.FROMFF);
            outgoingMsg.setSourceVertexId(getVertexId());
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
        posIterator = value.getFRList().iterator(); // FRList
        while(posIterator.hasNext()){
            outgoingMsg.setFlag(AdjMessage.FROMFR);
            outgoingMsg.setSourceVertexId(getVertexId());
            destVertexId.set(posIterator.next());
            sendMsg(destVertexId, outgoingMsg);
        }
    }
    
    /**
     * start sending message
     */
    public void startSendMsg() {
        if (VertexUtil.isHeadVertexWithIndegree(getVertexValue())) {
            outgoingMsg.setFlag((byte)(MessageFlag.IS_HEAD));//| MessageFlag.HEAD_SHOULD_MERGEWITHNEXT
            sendMsgToAllNextNodes(getVertexValue());
            voteToHalt();
        }
        if (VertexUtil.isRearVertexWithOutdegree(getVertexValue())) {
            outgoingMsg.setFlag((byte)(MessageFlag.IS_HEAD));//| MessageFlag.HEAD_SHOULD_MERGEWITHPREV
            sendMsgToAllPreviousNodes(getVertexValue());
            voteToHalt();
        }
        if (VertexUtil.isHeadWithoutIndegree(getVertexValue())){
            outgoingMsg.setFlag((byte)(MessageFlag.IS_HEAD));//| MessageFlag.HEAD_SHOULD_MERGEWITHNEXT
            sendMsg(getVertexId(), outgoingMsg); //send to itself
            voteToHalt();
        }
        if (VertexUtil.isRearWithoutOutdegree(getVertexValue())){
            outgoingMsg.setFlag((byte)(MessageFlag.IS_HEAD));//| MessageFlag.HEAD_SHOULD_MERGEWITHPREV
            sendMsg(getVertexId(), outgoingMsg); //send to itself
            voteToHalt();
        }
    }

    /**
     * initiate head, rear and path node
     */
    public void initState(Iterator<MessageWritable> msgIterator) {
        while (msgIterator.hasNext()) {
            if (!VertexUtil.isPathVertex(getVertexValue())
                    && !VertexUtil.isHeadWithoutIndegree(getVertexValue())
                    && !VertexUtil.isRearWithoutOutdegree(getVertexValue())) {
                msgIterator.next();
                voteToHalt();
            } else {
                incomingMsg = msgIterator.next();
                if(getHeadFlag() > 0)
                    voteToHalt();
                else 
                    getVertexValue().setState(incomingMsg.getFlag());
            }
        }
    }
    
    /**
     * check if A need to be flipped with successor
     */
    public boolean ifFilpWithSuccessor(){
        if(getVertexValue().getFRList().getLength() > 0)
            return true;
        else
            return false;
    }
    
    /**
     * check if A need to be filpped with predecessor
     */
    public boolean ifFlipWithPredecessor(){
        if(getVertexValue().getRFList().getLength() > 0)
            return true;
        else
            return false;
    }
    
    /**
     * set adjMessage to successor(from predecessor)
     */
    public void setSuccessorAdjMsg(){
        outFlag &= MessageFlag.DIR_CLEAR;
        if(getVertexValue().getFFList().getLength() > 0)
            outFlag |= MessageFlag.DIR_FF;
        else if(getVertexValue().getFRList().getLength() > 0)
            outFlag |= MessageFlag.DIR_FR;
        else
            outFlag |= MessageFlag.DIR_NO;
    }
    
    /**
     * set adjMessage to predecessor(from successor)
     */
    public void setPredecessorAdjMsg(){
        outFlag &= MessageFlag.DIR_CLEAR;
        if(getVertexValue().getRFList().getLength() > 0)
            outFlag |= MessageFlag.DIR_RF;
        else if(getVertexValue().getRRList().getLength() > 0)
            outFlag |= MessageFlag.DIR_RR;
        else
            outFlag |= MessageFlag.DIR_NO;
    }
    
    /**
     * send update message to neighber
     * @throws IOException 
     */
    public void broadcastUpdateMsg(){
        if((getVertexValue().getState() & State.IS_HEAD) > 0)
            outFlag |= MessageFlag.IS_HEAD;
        switch(getVertexValue().getState() & State.SHOULD_MERGE_MASK){
            case State.SHOULD_MERGEWITHPREV:
                setSuccessorAdjMsg();
                if(ifFlipWithPredecessor())
                    outgoingMsg.setFlip(true);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getIncomingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                if(getNextDestVertexId(getVertexValue()) != null)
                    sendMsg(getNextDestVertexId(getVertexValue()), outgoingMsg);
                break;
            case State.SHOULD_MERGEWITHNEXT:
                setPredecessorAdjMsg();
                if(ifFilpWithSuccessor())
                    outgoingMsg.setFlip(true);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getOutgoingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                if(getPreDestVertexId(getVertexValue()) != null)
                    sendMsg(getPreDestVertexId(getVertexValue()), outgoingMsg);
                break; 
        }
    }
    
    /**
     * send update message to neighber for P2
     * @throws IOException 
     */
    public void sendUpdateMsg(){
        outgoingMsg.setUpdateMsg(true);
        byte meToNeighborDir = (byte) (incomingMsg.getFlag() & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);
        switch(neighborToMeDir){
            case MessageFlag.DIR_FF:
            case MessageFlag.DIR_FR:
                sendUpdateMsgToPredecessor();
                break;
            case MessageFlag.DIR_RF:
            case MessageFlag.DIR_RR: 
                sendUpdateMsgToSuccessor();
                break;
        }
    }
    
    /**
     * send merge message to neighber for P2
     * @throws IOException 
     */
    public void sendMergeMsg(){
        outgoingMsg.setUpdateMsg(false);
        if(selfFlag == State.IS_HEAD){
            byte newState = getVertexValue().getState(); 
            newState &= ~State.IS_HEAD;
            newState |= State.IS_OLDHEAD;
            getVertexValue().setState(newState);
            resetSelfFlag();
            outFlag |= MessageFlag.IS_HEAD;  
        } else if(selfFlag == State.IS_OLDHEAD){
            outFlag |= MessageFlag.IS_OLDHEAD;
            voteToHalt();
        }
        byte meToNeighborDir = (byte) (incomingMsg.getFlag() & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);
        switch(neighborToMeDir){
            case MessageFlag.DIR_FF:
            case MessageFlag.DIR_FR:
                setSuccessorAdjMsg();
                if(ifFlipWithPredecessor())
                    outgoingMsg.setFlip(true);
                else
                    outgoingMsg.setFlip(false);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getIncomingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                outgoingMsg.setAcutalKmer(getVertexValue().getKmer());
                sendMsg(incomingMsg.getSourceVertexId(), outgoingMsg); //getNextDestVertexId(getVertexValue())
                break;
            case MessageFlag.DIR_RF:
            case MessageFlag.DIR_RR:
                setPredecessorAdjMsg();
                if(ifFilpWithSuccessor())
                    outgoingMsg.setFlip(true);
                else
                    outgoingMsg.setFlip(false);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getOutgoingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                outgoingMsg.setAcutalKmer(getVertexValue().getKmer());
                sendMsg(incomingMsg.getSourceVertexId(), outgoingMsg); //getPreDestVertexId(getVertexValue())
                break; 
        }
    }
    
    /**
     * send final merge message to neighber for P2
     * @throws IOException 
     */
    public void sendFinalMergeMsg(){
        outFlag |= MessageFlag.IS_FINAL;
        byte meToNeighborDir = (byte) (incomingMsg.getFlag() & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);
        switch(neighborToMeDir){
            case MessageFlag.DIR_FF:
            case MessageFlag.DIR_FR:
                setSuccessorAdjMsg();
                if(ifFlipWithPredecessor())
                    outgoingMsg.setFlip(true);
                else
                    outgoingMsg.setFlip(false);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getIncomingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                outgoingMsg.setAcutalKmer(getVertexValue().getKmer());
                sendMsg(incomingMsg.getSourceVertexId(), outgoingMsg); //getNextDestVertexId(getVertexValue())
                break;
            case MessageFlag.DIR_RF:
            case MessageFlag.DIR_RR:
                setPredecessorAdjMsg();
                if(ifFilpWithSuccessor())
                    outgoingMsg.setFlip(true);
                else
                    outgoingMsg.setFlip(false);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getOutgoingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                outgoingMsg.setAcutalKmer(getVertexValue().getKmer());
                sendMsg(incomingMsg.getSourceVertexId(), outgoingMsg); //getPreDestVertexId(getVertexValue())
                break; 
        }
    }
    
    /**
     * send merge message to neighber for P4
     * @throws IOException 
     */
    public void broadcastMergeMsg(){
        if(headFlag > 0)
            outFlag |= MessageFlag.IS_HEAD;
        switch(getVertexValue().getState() & State.SHOULD_MERGE_MASK) {
            case State.SHOULD_MERGEWITHNEXT:
                setSuccessorAdjMsg();
                if(ifFlipWithPredecessor())
                    outgoingMsg.setFlip(true);
                else
                    outgoingMsg.setFlip(false);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getIncomingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                outgoingMsg.setAcutalKmer(getVertexValue().getKmer());
                sendMsg(getNextDestVertexId(getVertexValue()), outgoingMsg);
                deleteVertex(getVertexId());
                break;
            case State.SHOULD_MERGEWITHPREV:
                setPredecessorAdjMsg();
                if(ifFilpWithSuccessor())
                    outgoingMsg.setFlip(true);
                else
                    outgoingMsg.setFlip(false);
                outgoingMsg.setFlag(outFlag);
                outgoingMsg.setNeighberNode(getVertexValue().getOutgoingList());
                outgoingMsg.setSourceVertexId(getVertexId());
                outgoingMsg.setAcutalKmer(getVertexValue().getKmer());
                sendMsg(getPreDestVertexId(getVertexValue()), outgoingMsg);
                deleteVertex(getVertexId());
                break; 
        }
    }
    
    public void setStateAsMergeWithNext(){
    	byte state = getVertexValue().getState();
    	state &= State.SHOULD_MERGE_CLEAR;
        state |= State.SHOULD_MERGEWITHNEXT;
        getVertexValue().setState(state);
    }
    
    /**
     * This vertex tries to merge with next vertex and send update msg to neighber
     * @throws IOException 
     */
    public void sendUpdateMsgToPredecessor(){
    	if(hasNextDest(getVertexValue())){
    	    setStateAsMergeWithNext();
		    broadcastUpdateMsg();
    	}
    }
    
    public void setStateAsMergeWithPrev(){
        byte state = getVertexValue().getState();
        state &= State.SHOULD_MERGE_CLEAR;
        state |= State.SHOULD_MERGEWITHPREV;
        getVertexValue().setState(state);
    }
    
    /**
     * This vertex tries to merge with next vertex and send update msg to neighber
     * @throws IOException 
     */
    public void sendUpdateMsgToSuccessor(){
    	if(hasPrevDest(getVertexValue())){
    	    setStateAsMergeWithPrev();
		    broadcastUpdateMsg();
    	}
    }
    
    /**
     * set state as no_merge
     */
    public void setStateAsNoMerge(){
    	byte state = getVertexValue().getState();
    	//state |= State.SHOULD_MERGE_CLEAR;
        state |= State.NO_MERGE;
        getVertexValue().setState(state);
    }
    
    /**
     * Returns the edge dir for B->A when the A->B edge is type @dir
     */
    public byte mirrorDirection(byte dir) {
        switch (dir) {
            case MessageFlag.DIR_FF:
                return MessageFlag.DIR_RR;
            case MessageFlag.DIR_FR:
                return MessageFlag.DIR_FR;
            case MessageFlag.DIR_RF:
                return MessageFlag.DIR_RF;
            case MessageFlag.DIR_RR:
                return MessageFlag.DIR_FF;
            default:
                throw new RuntimeException("Unrecognized direction in flipDirection: " + dir);
        }
    }
    
    /**
     * check if need filp
     */
    public byte flipDirection(byte neighborDir, boolean flip){
        if(flip){
            switch (neighborDir) {
                case MessageFlag.DIR_FF:
                    return MessageFlag.DIR_FR;
                case MessageFlag.DIR_FR:
                    return MessageFlag.DIR_FF;
                case MessageFlag.DIR_RF:
                    return MessageFlag.DIR_RR;
                case MessageFlag.DIR_RR:
                    return MessageFlag.DIR_RF;
                default:
                    throw new RuntimeException("Unrecognized direction for neighborDir: " + neighborDir);
            }
        } else 
            return neighborDir;
    }
    
    /**
     * updateAdjList
     */
    public void processUpdate(){
        inFlag = incomingMsg.getFlag();
        byte meToNeighborDir = (byte) (inFlag & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);
        
        byte neighborToMergeDir = flipDirection(neighborToMeDir, incomingMsg.isFlip());
        
        getVertexValue().processUpdates(neighborToMeDir, incomingMsg.getSourceVertexId(), 
                neighborToMergeDir, VertexUtil.getNodeIdFromAdjacencyList(incomingMsg.getNeighberNode()));
    }
    
    /**
     * merge and updateAdjList merge with one neighbor
     */
    public void processMerge(){
        inFlag = incomingMsg.getFlag();
        byte meToNeighborDir = (byte) (inFlag & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);
        
        if((inFlag & MessageFlag.IS_HEAD) > 0){
            byte state = getVertexValue().getState();
            state |= State.IS_HEAD;
            getVertexValue().setState(state);
        }
        
        byte neighborToMergeDir = flipDirection(neighborToMeDir, incomingMsg.isFlip());
        
        getVertexValue().processMerges(neighborToMeDir, incomingMsg.getSourceVertexId(), 
                neighborToMergeDir, VertexUtil.getNodeIdFromAdjacencyList(incomingMsg.getNeighberNode()),
                kmerSize, incomingMsg.getActualKmer());
    }
    
    /**
     * merge and updateAdjList  having parameter
     */
    public void processMerge(MessageWritable msg){
        byte meToNeighborDir = (byte) (msg.getFlag() & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);

        byte neighborToMergeDir = flipDirection(neighborToMeDir, msg.isFlip());
        
        getVertexValue().processMerges(neighborToMeDir, msg.getSourceVertexId(), 
                neighborToMergeDir, VertexUtil.getNodeIdFromAdjacencyList(msg.getNeighberNode()),
                kmerSize, msg.getActualKmer());
    }
    
    /**
     * final merge and updateAdjList  having parameter for p2
     */
    public void processFinalMerge(MessageWritable msg){
        byte meToNeighborDir = (byte) (msg.getFlag() & MessageFlag.DIR_MASK);
        byte neighborToMeDir = mirrorDirection(meToNeighborDir);

        byte neighborToMergeDir = flipDirection(neighborToMeDir, msg.isFlip());
        
        String selfString;
        String match;
        String msgString;
        int index;
        switch(neighborToMeDir){
            case MessageFlag.DIR_FF:
                selfString = getVertexValue().getKmer().toString();
                match = selfString.substring(selfString.length() - kmerSize + 1,selfString.length()); 
                msgString = msg.getActualKmer().toString();
                index = msgString.indexOf(match);
                tmpKmer.reset(msgString.length() - index);
                tmpKmer.setByRead(msgString.substring(index).getBytes(), 0);
                break;
            case MessageFlag.DIR_FR:
                selfString = getVertexValue().getKmer().toString();
                match = selfString.substring(selfString.length() - kmerSize + 1,selfString.length()); 
                msgString = GeneCode.reverseComplement(msg.getActualKmer().toString());
                index = msgString.indexOf(match);
                tmpKmer.reset(msgString.length() - index);
                tmpKmer.setByReadReverse(msgString.substring(index).getBytes(), 0);
                break;
            case MessageFlag.DIR_RF:
                selfString = getVertexValue().getKmer().toString();
                match = selfString.substring(0,kmerSize - 1); 
                msgString = GeneCode.reverseComplement(msg.getActualKmer().toString());
                index = msgString.lastIndexOf(match) + kmerSize - 2;
                tmpKmer.reset(index + 1);
                tmpKmer.setByReadReverse(msgString.substring(0, index + 1).getBytes(), 0);
                break;
            case MessageFlag.DIR_RR:
                selfString = getVertexValue().getKmer().toString();
                match = selfString.substring(0,kmerSize - 1); 
                msgString = msg.getActualKmer().toString();
                index = msgString.lastIndexOf(match) + kmerSize - 2;
                tmpKmer.reset(index + 1);
                tmpKmer.setByRead(msgString.substring(0, index + 1).getBytes(), 0);
                break;
        }
       
        getVertexValue().processMerges(neighborToMeDir, msg.getSourceVertexId(), 
                neighborToMergeDir, VertexUtil.getNodeIdFromAdjacencyList(msg.getNeighberNode()),
                kmerSize, tmpKmer);
    }
    
    /**
     * set head state
     */
    public void setHeadState(){
        byte state = getVertexValue().getState();
        state &= State.VERTEX_CLEAR;
        state |= State.IS_HEAD;
        getVertexValue().setState(state);
    }
    
    /**
     * set final state
     */
    public void setFinalState(){
        byte state = getVertexValue().getState();
        state &= State.VERTEX_CLEAR;
        state |= State.IS_FINAL;
        getVertexValue().setState(state);
    }
    
    /**
     * set stop flag
     */
    public void setStopFlag(){
        byte state = getVertexValue().getState();
        state &= State.VERTEX_CLEAR;
        state |= State.IS_FINAL;
        getVertexValue().setState(state);
    }
    
    /**
     * get Vertex state
     */
    public byte getMsgFlag(){
        return (byte)(incomingMsg.getFlag() & MessageFlag.VERTEX_MASK);
    }
    
    /**
     * reset selfFlag
     */
    public void resetSelfFlag(){
        selfFlag =(byte)(getVertexValue().getState() & MessageFlag.VERTEX_MASK);
    }
    
    /**
     * broadcast kill self to all neighbers  Pre-condition: vertex is a path vertex 
     */
    public void broadcaseKillself(){
        outFlag = 0;
        outFlag |= MessageFlag.KILL;
        outFlag |= MessageFlag.DIR_FROM_DEADVERTEX;
        outgoingMsg.setSourceVertexId(getVertexId());
        
        if(getVertexValue().getFFList().getCountOfPosition() > 0){//#FFList() > 0
            outFlag |= MessageFlag.DIR_FF;
            outgoingMsg.setFlag(outFlag);
            sendMsg(getVertexValue().getFFList().getPosition(0), outgoingMsg);
        }
        else if(getVertexValue().getFRList().getCountOfPosition() > 0){//#FRList() > 0
            outFlag |= MessageFlag.DIR_FR;
            outgoingMsg.setFlag(outFlag);
            sendMsg(getVertexValue().getFRList().getPosition(0), outgoingMsg);
        }
        
        
        if(getVertexValue().getRFList().getCountOfPosition() > 0){//#RFList() > 0
            outFlag |= MessageFlag.DIR_RF;
            outgoingMsg.setFlag(outFlag);
            sendMsg(getVertexValue().getRFList().getPosition(0), outgoingMsg);
        }
        else if(getVertexValue().getRRList().getCountOfPosition() > 0){//#RRList() > 0
            outFlag |= MessageFlag.DIR_RR;
            outgoingMsg.setFlag(outFlag);
            sendMsg(getVertexValue().getRRList().getPosition(0), outgoingMsg);
        }
        
        deleteVertex(getVertexId());
    }
    
    /**
     * do some remove operations on adjMap after receiving the info about dead Vertex
     */
    public void responseToDeadVertex(){
        switch(incomingMsg.getFlag() & MessageFlag.DIR_MASK){
            case MessageFlag.DIR_FF:
                //remove incomingMsg.getSourceId from RR positionList
                posIterator = getVertexValue().getRRList().iterator();
                while(posIterator.hasNext()){
                    tmpKmer = posIterator.next();
                    if(tmpKmer.equals(incomingMsg.getSourceVertexId())){
                        posIterator.remove();
                        break;
                    }
                }
                break;
            case MessageFlag.DIR_FR:
                //remove incomingMsg.getSourceId from FR positionList
                posIterator = getVertexValue().getFRList().iterator();
                while(posIterator.hasNext()){
                    tmpKmer = posIterator.next();
                    if(tmpKmer.equals(incomingMsg.getSourceVertexId())){
                        posIterator.remove();
                        break;
                    }
                }
                break;
            case MessageFlag.DIR_RF:
                //remove incomingMsg.getSourceId from RF positionList
                posIterator = getVertexValue().getRFList().iterator();
                while(posIterator.hasNext()){
                    tmpKmer = posIterator.next();
                    if(tmpKmer.equals(incomingMsg.getSourceVertexId())){
                        posIterator.remove();
                        break;
                    }
                }
                break;
            case MessageFlag.DIR_RR:
                //remove incomingMsg.getSourceId from FF positionList
                posIterator = getVertexValue().getFFList().iterator();
                while(posIterator.hasNext()){
                    tmpKmer = posIterator.next();
                    if(tmpKmer.equals(incomingMsg.getSourceVertexId())){
                        posIterator.remove();
                        break;
                    }
                }
                break;
        }
    }
    
    public boolean isKillMsg(){
        byte killFlag = (byte) (incomingMsg.getFlag() & MessageFlag.KILL_MASK);
        return killFlag == MessageFlag.KILL;
    }
    
    @Override
    public void compute(Iterator<MessageWritable> msgIterator) {
    }
}